from Products.CMFPlone.utils import _createObjectByType


TEST_CONTENT = [
    {'id':'doc1',
     'type':'Document',
     'Subject':['Apple','Banana','Orange','Lemon'],
    },
    {'id':'doc2',
     'type':'Document',
     'Subject':['Apple','Banana','Orange'],
    },
    {'id':'doc3',
     'type':'Document',
     'Subject':['Apple','Banana','Lemon'],
    },
    {'id':'doc4',
     'type':'Document',
     'Subject':['Apple','Banana','Lemon'],
    },
    {'id':'doc5',
     'type':'Document',
     'Subject':['Apple'],
    },
    {'id':'news1',
     'type':'News Item',
     'Subject':['Apple','Banana','Lemon'],
    },
    {'id':'news2',
     'type':'News Item',
     'Subject':['Apple','Banana','Orange','Lemon'],
    },
    {'id':'news3',
     'type':'News Item',
     'Subject':['Apple','Lemon'],
    },
    {'id':'news4',
     'type':'News Item',
     'Subject':['Apple','Banana','Orange','Lemon'],
    },
    {'id':'news5',
     'type':'News Item',
     'Subject':['Apple'],
    },
]


def construct_dummy_content(portal):
    #Create a container to keep it away from portal root
    _createObjectByType(type_name='Folder',
                        id='dummy_container',
                        container=portal)
    container = portal.dummy_container

    for item in TEST_CONTENT:
        _createObjectByType(type_name=item['type'],
                            id=item['id'],
                            container=container,
                            title=item['id'])
        obj = container.get(item['id'])
        obj.setSubject(item['Subject'])
        obj.reindexObject()

    _createObjectByType(type_name='Topic',
                        container=portal,
                        id='collection')

    collection = portal.collection
    type_crit = collection.addCriterion('Type','ATPortalTypeCriterion')
    type_crit.setValue(['News Item','Page'])
    sort_crit = collection.addCriterion('created','ATSortCriterion')
    collection.setSortCriterion('created', True)


from zope.interface import Interface

class IFacetBuilder(Interface):
    """Tool"""
    
    def facet_data(facets):
        """ See adapters.FacetBuilder
        """
    
    def add_weight_data(facetobjects):
        """ See adapters.FacetBuilder
        """
        
    def add_brains():
        """ See adapters.FacetBuilder
        """
        
    def facet_query(**kw):
        """ Return a dict with info about a selection of this facet. kw is the index to search for, ie Type='Document'
        """
        
class IFacetBaseView(Interface):
    """ Base view for anything regular BrowserView class that handles facets.
        Populates the request with facet brains and search data.
    """


"""This is an integration "unit" test. It uses PloneTestCase because I'm lazy
"""
import unittest
from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase import PloneTestCase as ptc
from collective.facetsupport.tests import base

#Imports for tests
from zope.publisher.browser import TestRequest
from collective.facetsupport.interfaces import IFacetBuilder
from collective.facetsupport.utilities import FacetBuilder
from collective.facetsupport.config import DEFAULT_INDEXES

ptc.setupPloneSite()

class TestIntegration(base.TestCase):
    """Test integration
    """
    #layer = contentstructure
    
    def afterSetUp(self):
        """This method is called before each single test. It can be used to
        set up common state. Setup that is specific to a particular test 
        should be done in that test method.
        """
        super(TestIntegration, self).afterSetUp()
            
        self.catalog = self.portal.portal_catalog

    def beforeTearDown(self):
        """This method is called after each single test. It can be used for
        cleanup, if you need it. Note that the test framework will roll back
        the Zope transaction at the end of each test, so tests are generally
        independent of one another. However, if you are modifying external
        resources (say a database) or globals (such as registering a new
        adapter in the Component Architecture during a test), you may want to
        tear things down here.
        """
        
    def testFacetBuilderIntegration(self):
        request = TestRequest()
        adapter = IFacetBuilder(self.portal, request)
        
        self.failUnless(adapter,'FacetBuilder not found')
        self.failUnless(IFacetBuilder.providedBy(adapter), 'IFacetBuilder not provided by FacetBuilder')
        self.failUnless(IFacetBuilder.implementedBy(FacetBuilder), "FacetBuilder doesn't implement IFacetBuilder")

    
    def testDummyStructure(self):
        doc = self.portal.dummy_container.get('doc1',None)
        self.failUnless(doc)
        self.assertEquals(doc.Subject(),('Apple','Banana','Orange','Lemon'))
        collection = self.portal.get('collection',None)
        self.failUnless(collection)
        #first-page is also a page, so 11 is right! :)
        self.assertEquals(len(collection.queryCatalog()),11)
        
    def testDummyInCatalog(self):
        self.failUnless(self.catalog(Subject='Apple'))
        self.assertEquals(len(self.catalog()),20)
    
    def testInlineFacetResultView(self):
        view = self.portal.restrictedTraverse('@@inline_facet_result',None)
        self.failUnless(view)

    def testInlineFacetMenuView(self):
        view = self.portal.restrictedTraverse('@@inline_facet_menu',None)
        self.failUnless(view)

def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """ 
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestIntegration))
    return suite



from Products.Five.browser import BrowserView
from zope.interface import implements

from collective.facetsupport.interfaces import IFacetBuilder, IFacetBaseView
from collective.facetsupport.config import DEFAULT_INDEXES
from ZTUtils.Zope import make_query
from Products.CMFCore.utils import getToolByName

import logging
logger = logging.getLogger('collective.facetsupport')

class FacetBaseView(BrowserView):
    """ Base view for anything regular BrowserView class that handles facets.
        Populates the request with facet brains and search data.
    """
    implements(IFacetBaseView)
    
    def __init__(self, context, request):
        super(FacetBaseView, self).__init__(context, request)
        fb = IFacetBuilder(context)
        #FIXME: This should be configurable in the control panel, and easily overridden in templates.
        #If it's added here that won't be possible
        self.use_indexes = DEFAULT_INDEXES
        
        #The adapter returns None if no update of the request is needed.
        data = fb.facet_data(self.use_indexes)
        if data is not None:
            request.facetsupport = data
        
        fb.add_weight_data(request.facetsupport['facets'])
        
        self.fb = fb
        self.catalog = getToolByName(context,'portal_catalog')

    @property
    def facets(self):
        return self.request.facetsupport['facets']
    
    @property
    def query(self):
        return self.request.facetsupport['query']

    def normalize_css(self, text):
        return self.fb.normalize_css(text)

    def query_link(self, remove=None):
        """ A link for a query. Remove keyword in remove.
        """
        query = self.query.copy()
        if remove in query:
            del query[remove]
        return make_query(query)

    #FIXME: This should be a part of the request instead. Really... :)
    #FIXME: Template name could be a part of the request too
    def link_for_term(self, facet, term):
        query = self.fb.facet_query(**{facet:term})
        return "?%s" % make_query(query)

class InlineFacetResultView(FacetBaseView):
    """ Inline view facet results.
    """

class InlineFacetMenuView(FacetBaseView):
    """ Inline view for facet menu.
    """
    




from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.1a'
long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '==============\n'
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    read('CONTRIBUTORS.txt')
    + '\n' +
    'Known Issues\n'
    '============\n'
    + '\n' +
    read('KNOWN_ISSUES.txt')
)

setup(name='collective.facetsupport',
      version=version,
      description="Tools for generating facets for content in Plone",
      long_description=long_description,
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        ],
      keywords='plone',
      author='Robin Harms Oredsson',
      author_email='robin@betahaus.net',
      url='https://svn.plone.org/svn/collective/collective.facetsupport',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from collective.facetsupport.interfaces import IFacetBuilder
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import getUtility
from Products.CMFPlone.utils import safe_unicode

import logging
logger = logging.getLogger('collective.facetsupport')

class FacetBuilder(object):
    implements(IFacetBuilder)
    
    def __init__(self, context):
        self.context = context
        self.request = context.REQUEST
        self.catalog = getToolByName(context,'portal_catalog')
        self.catalog_indexes = self.catalog.indexes()
        self.normalizer = getUtility(IIDNormalizer)

        #Check context for usable request method. Plone root has a Python FS script called queryCatalog which we don't want!
        if self.context.meta_type != 'Plone Site' and shasattr(context, 'queryCatalog'):
            self.query_method = context.queryCatalog
        else:
            #We might want to update this
            self.query_method = context.getFolderContents
    
    def _construct_facet_query(self):
        """ Construct a catalog query from a request.
            Only fetches things from the request object if it exists as a catalog index.
            Most of the time, you'' want to fetch this data from request.facetsupport['query']
        """
        reqkeys = self.request.keys()
        indexes = self.catalog_indexes
        query = {}
        for index in indexes:
            if index in reqkeys:
                query[index] = self.request[index]
        
        return query
    
    def normalize_css(self, text):
        """ Fix css classes or ids so they're okay with i18n chars.
        """
        if type(text) != unicode:
            text = text.decode('utf-8')
        return self.normalizer.normalize(text)


    def facet_data(self, facets):
        """ facets are facets to use. This equals catalog indexes.
            Layout of the request.facetsupport property
            
            facet: list of FacetData objects. See FacetData below.
            
            query: The query to pass along to get this result.
            
            SearchableText: The SearchableText query.
            
            This should in most cases be set as the request.facetsupport property
            
            Only executes if facetsupport doesn't exist in the request, otherwise it returns None
        """
        
        context = self.context
        request = self.request
        
        fs  = {}
        
        fs['query'] = self._construct_facet_query()
        #fs['brains'] = self.query_method(**fs['query'])
        fs['facets'] = []

        catalog_indexes = self.catalog.indexes()
        for facet in facets:
            if facet in catalog_indexes:
                val = list(self.catalog.uniqueValuesFor(facet))
                val.sort()
            else:
                facet = u'Index not found: %s' % facet 
                val = []
            #FIXME: This should handle UIDs that are relations too
            obj = FacetData(facet)
            obj.terms = val
            fs['facets'].append(obj)
            
        #SearchableText is a special case - it should always be there
        search = getattr(request,'SearchableText',None)
        if search:
            fs['SearchableText'] = search
        
        return fs

    def add_weight_data(self, facetobjects):
        """ Updates facetobjects with weight
        """
        query = self.request.facetsupport['query']
        for obj in facetobjects:
            obj.weight = self._per_facet_weight(obj.facet, obj.terms, query)


    def _per_facet_weight(self, facet, terms, query):
        """ Returns a dict with layout {item:weight,...}
            query is the base query that's active right now.
        """
        result = {}
        for item in terms:
            thisq = query.copy()
            thisq.update({facet:item})
            result[item] = len( self.query_method(thisq) )
        return result

    def add_brains(self):
        """ Add brains to the existing request """
        fs = self.request.get('facetsupport',None)
        if fs is None:
            return None
        fs['brains'] = self.query_method(**fs['query'])
        return True

    def facet_query(self, **kw):
        """ Returns a dict with the query updated with this item selected.
            Used for constructing links or calculating weight of a facet.
        """
        if kw is None or not hasattr(self.request, 'facetsupport'):
            return None

        query = self.request.facetsupport['query'].copy()

        for k in kw:
            #TODO: Should we allow anything here?
            if k in self.catalog_indexes:
                #This will replace any existing keys in the dict, which is the point
                query[k] = kw[k]
        return query 

class FacetData(object):
    """ This object is used to fetch results for each facet.
        facet: The name of this facet. Must be the same as the catalog index
        terms: The different terms in this facet
        weight: weight for each term
        
    """
    facet = ''
    terms = []
    weight = {}
    #FIXME: Implement - Some facets can have uids as terms. In that case they need to be handled in a special way.
    reference_terms = False
    
    def __init__(self, facet):
        setattr(self, 'facet', facet)

    @property
    def title(self):
        """Just to make skinning a bit easier :) """
        return self.facet


"""
    Functional tests - collects all *.txt files
"""

import os
import glob
from zope.testing import doctest
import unittest
from Globals import package_home
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from base import FunctionalTestCase

GLOBALS = globals()


UNITTESTS = []

OPTIONFLAGS = (doctest.ELLIPSIS | 
               doctest.NORMALIZE_WHITESPACE | 
               doctest.REPORT_ONLY_FIRST_FAILURE)

    
def list_doctests():
    home = package_home(GLOBALS)
    return [filename for filename in
            glob.glob(os.path.sep.join([home, '*.txt']))
            if os.path.basename(filename) not in UNITTESTS]

def test_suite():
    filenames = list_doctests()

    test_suites = [Suite(os.path.basename(filename),
                                  optionflags=OPTIONFLAGS,
                                  package='collective.facetsupport.tests',
                                  test_class=FunctionalTestCase)
                                  for filename in filenames]

    return unittest.TestSuite(test_suites)

